/*
 * ImageInformation.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import java.io.*;
import java.net.*;
import misao.util.*;

public class ImageInformation implements Printable {
    public String image_url;
    public Size image_size;
    public JulianDay exposure_start;
    public double exposure_seconds;
    public CoorMapping center_coor;
    public double field_of_view;
    public double pa;
    public DistortionField distortion_field;
    public Coor pos_error;	// in arcsec
    public MagError mag_error;
    public double limiting_mag;
    public double faintest_mag;
    public String mag_code;

    public ImageInformation ( ) {
	image_url = "";
	image_size = new Size();
	exposure_start = new JulianDay();
	exposure_seconds = 0.0;
	center_coor = new CoorMapping();
	field_of_view = 0.0;
	pa = 0.0;
	distortion_field = new DistortionField();
	pos_error = new Coor();
	mag_error = new MagError(0.0);
	limiting_mag = 20.0;
	faintest_mag = 20.0;
	mag_code = "";
    }

    public void setParameter ( Parameter p ) throws Exception {
	if (p.getKey().equals("image")) {
	    image_url = p.getValue();
	} else if (p.getKey().equals("image_size")) {
	    String s = p.getValue();
	    int q = s.indexOf('x');
	    image_size = new Size(Integer.parseInt(s.substring(0,q).trim()), Integer.parseInt(s.substring(q+1).trim()));
	} else if (p.getKey().equals("exposure_start")) {
	    exposure_start = JulianDay.createJulianDay(p.getValue());
	} else if (p.getKey().equals("exposure")) {
	    exposure_seconds = Astro.doubleSecondValueOf(p.getValue());
	} else if (p.getKey().equals("center")) {
	    center_coor = new CoorMapping(Coor.createCoor(p.getValue()));
	} else if (p.getKey().equals("field_of_view")) {
	    String s = p.getValue();
	    int q = s.indexOf('x');
	    String sw = s.substring(0,q).trim();
	    String sh = s.substring(q+1).trim();
	    q = sh.indexOf(' ');
	    if (q < 0)
		field_of_view = Astro.doubleValueOf(sw);
	    else
		field_of_view = Astro.doubleDegreeValueOf(sw + sh.substring(q));
	} else if (p.getKey().equals("p.a.")) {
	    pa = Astro.doubleDegreeValueOf(p.getValue());
	} else if (p.getKey().equals("limiting_mag")) {
	    limiting_mag = Astro.doubleValueOf(p.getValue());
	} else if (p.getKey().equals("faintest_mag")) {
	    faintest_mag = Astro.doubleValueOf(p.getValue());
	} else if (p.getKey().equals("pos_error")) {
	    String s = p.getValue();
	    int q1 = s.indexOf('"');
	    int q2 = s.lastIndexOf('"');
	    pos_error = new Coor(Astro.doubleValueOf(s.substring(0,q1)), Astro.doubleValueOf(s.substring(q1+1,q2)));
	} else if (p.getKey().equals("mag_error")) {
	    mag_error = MagError.createMagError(p.getValue());
	} else if (p.getKey().equals("mag_code")) {
	    mag_code = p.getValue();
	}
    }

    public void print ( PrintStream ps ) {
	new Parameter("image", image_url).print(ps);
	new Parameter("image_size", "" + image_size.getWidth() + " x " + image_size.getHeight()).print(ps);
	new Parameter("exposure_start", exposure_start.toString()).print(ps);
	new Parameter("exposure", "" + exposure_seconds + " sec").print(ps);
	new Parameter("center", center_coor.toStringRA() + " " + center_coor.toStringDecl()).print(ps);
	double fov_width = field_of_view;
	double fov_height = field_of_view * (double)image_size.getHeight() / (double)image_size.getWidth();
	double fov_tmp = fov_width;
	if (fov_height > fov_tmp)
	    fov_tmp = fov_height;
	if (fov_tmp < 1.0 / 60.0) {
	    new Parameter("field_of_view", Astro.toString(fov_width * 3600.0, 6, 3).trim() + " x " + Astro.toString(fov_height * 3600.0, 6, 3).trim() + " arcsec").print(ps);
	} else if (fov_tmp < 10.0 / 60.0) {
	    new Parameter("field_of_view", Astro.toString(fov_width * 60.0, 6, 2).trim() + " x " + Astro.toString(fov_height * 60.0, 6, 2).trim() + " arcmin").print(ps);
	} else if (fov_tmp < 1.0) {
	    new Parameter("field_of_view", Astro.toString(fov_width * 60.0, 6, 3).trim() + " x " + Astro.toString(fov_height * 60.0, 6, 3).trim() + " arcmin").print(ps);
	} else if (fov_tmp < 10.0) {
	    new Parameter("field_of_view", Astro.toString(fov_width, 6, 2).trim() + " x " + Astro.toString(fov_height, 6, 2).trim() + " deg").print(ps);
	} else {
	    new Parameter("field_of_view", Astro.toString(fov_width, 6, 3).trim() + " x " + Astro.toString(fov_height, 6, 3).trim() + " deg").print(ps);
	}
	new Parameter("pixel_size", Astro.toString(field_of_view * 3600.0 / (double)image_size.getWidth(), 8, 5).trim() + "\" / pixel").print(ps);
	new Parameter("p.a.", Astro.toString(pa, 6, 3).trim() + " deg").print(ps);
	new Parameter("limiting_mag", Astro.toString(limiting_mag, 5, 2).trim() + " mag").print(ps);
	new Parameter("faintest_mag", Astro.toString(faintest_mag, 5, 2).trim() + " mag").print(ps);
	new Parameter("pos_error", Astro.toString(pos_error.getRA(), 8, 5).trim() + "\" " + Astro.toString(pos_error.getDecl(), 8, 5).trim() + "\"").print(ps);
	new Parameter("mag_error", mag_error.toString()).print(ps);
	if (mag_code.length() > 0)
	    new Parameter("mag_code", mag_code).print(ps);
    }
}
