/*
 * Astro.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.*;

public class Astro {
    public static final double RAD = 0.01745329251994329547;	// pi / 180

    public static final double JULIUS_1850 = 2396758.0;
    public static final double JULIUS_1900 = 2415020.0;
    public static final double JULIUS_2000 = 2451545.0;
    public static final double BESSELL_1950 = 2433282.42346;
    public static final double EPSILON = 23.4392911;		// ecliptic inclination at 2000.0
    public static final double GAUSS_K = 0.01720209895;		// Gaussian constant
    public static final double SOLAR_YEAR = 365.242190;		// one solar year
    public static final double LIGHT_DELAY = 499.004782;
    public static final double DK_per_DS = 1.0027379093;	// mean sidereal hour / mean solar hour
    public static final double SIDEREAL_DAY = 359.0170438;	// one sidereal day

    public static final double ILLEGAL_MAG = 99.9;
    public static final double MAG_STEP = 2.5;

    public static final double SBIG_RATIO = 1.176;

    public static double norm(double x, double y) {
	if (x >= 0.0)
	    return (x) - (double)((long)((x)/(y))*(y));
	else
	    return (x) - (double)((long)((x)/(y))*(y)) + (y);
    }

    public static final double sinh(double x) {
	return (Math.exp(x) - Math.exp(- x)) / 2.0;
    }

    public static final double cosh(double x) {
	return (Math.exp(x) + Math.exp(- x)) / 2.0;
    }

    public static double get_t1850(double jde) {
	return ((jde) - JULIUS_1850) / 36525.0;
    }

    public static double get_t1900(double jde) {
	return ((jde) - JULIUS_1900) / 36525.0;
    }

    public static double get_t2000(double jde) {
	return ((jde) - JULIUS_2000) / 36525.0;
    }

    public static double epsilon2000(double t) {
	return 23.4392911-0.01300417*(t)-0.00000164*(t)*(t)+0.0000005036*(t)*(t)*(t);
    }

    public static double greenwich_zero2000(double t) {
	return 100.460618375+36000.7700537*(t)+0.000387933*(t)*(t)-0.00000002583*(t)*(t)*(t);
    }

    public static double angleDistance ( double angle1, double angle2 ) {
	double dt = Math.abs(angle2 - angle1);
	if (dt > 180.0)
	    dt = 360.0 - dt;
	return dt;
    }

    public static double angleMean ( double angle1, double angle2 ) {
	return angleMean(angle1, angle2, 1.0, 1.0);
    }

    public static double angleMean ( double angle1, double angle2, double weight1, double weight2 ) {
	double dt = Math.abs(angle2 - angle1);
	if (dt > 180.0) {
	    angle1 += 360.0;
	    dt = Math.abs(angle2 - angle1);
	}
	if (dt > 180.0) {
	    angle1 -= 360.0 * 2.0;
	    dt = Math.abs(angle2 - angle1);
	}
	dt = angle2 - angle1;
	double angle = angle1 + dt * weight2 / (weight1 + weight2);
	return norm(angle, 360.0);
    }

    public static double getProperLimitingMagnitude ( double fov ) {
	return -2.367 * Math.log(fov) + 14.925;
    }

    public static double getUSNOVmag ( double magR, double magB ) {
	return magR + 0.375 * (magB - magR);
    }

    public static double blendMagnitude ( double mag1, double mag2 ) {
	double val = Math.pow(Astro.MAG_STEP, - mag1);
	val += Math.pow(Astro.MAG_STEP, - mag2);
	double mag = - Math.log(val) / Math.log(Astro.MAG_STEP);
	return mag;
    }

    public static int intValueOf ( byte[] b, int offset, int bytes ) {
	int value = 0;
	for (int i = 0 ; i < bytes ; i++) {
	    int v = b[offset + i];
            if (v < 0)
                v += 256;
	    value = (value << 8) | v;
	}
	return value;
    }

    public static double doubleValueOf ( String _s ) {
	String s = _s.trim();
	int p = s.indexOf(' ');
	if (p >= 0)
	    s = s.substring(0, p);

	p = s.indexOf('E');
	if (p >= 0  &&  p+1 < s.length()  &&  (s.charAt(p+1) == '+'  ||  s.charAt(p+1) == '-')) {
	    double f = Double.valueOf(s.substring(0,p)).doubleValue();
	    int q = Integer.parseInt(s.substring(p+2));
	    if (s.charAt(p+1) == '-')
		q = - q;

	    double a = 10.0;
	    if (q < 0) {
		a = 0.1;
		q = - q;
	    }
	    for (int i = 0 ; i < q ; i++)
		f *= a;

	    return f;
	}

	if (s.charAt(0) == '+')
	    s = s.substring(1);

	p = s.indexOf('.');
	int q = s.substring(p+1).indexOf('.');
	if (q >= 0)
	    s = s.substring(0,q);

	p = s.substring(1).indexOf('-');
	if (p >= 0)
	    s = s.substring(0,p);

	String s2 = "";
	for (int i = 0 ; i < s.length() ; i++) {
	    if (('0' <= s.charAt(i)  &&  s.charAt(i) <= '9')  ||  s.charAt(i) == '.'  ||  s.charAt(i) == '-')
		s2 += s.charAt(i);
	    else
		break;
	}

	return Double.valueOf(s2).doubleValue();
    }

    public static double doubleDegreeValueOf ( String _s ) {
	double f = doubleValueOf(_s);

	String s = _s.trim();
	int p = s.indexOf(' ');
	if (p >= 0) {
	    s = s.substring(p+1).trim();
	    if (s.length() >= 6  &&  s.substring(0, 6).equalsIgnoreCase("arcmin")) {
		f /= 60.0;
	    } else if (s.length() >= 6  &&  s.substring(0, 6).equalsIgnoreCase("arcsec")) {
		f /= 3600.0;
	    }
	}

	return f;
    }

    public static double doubleSecondValueOf ( String _s ) {
	double f = doubleValueOf(_s);

	String s = _s.trim();
	int p = s.indexOf(' ');
	if (p >= 0) {
	    s = s.substring(p+1).trim();
	    if (s.length() >= 3  &&  s.substring(0, 3).equalsIgnoreCase("min")) {
		f *= 60.0;
	    }
	}

	return f;
    }

    public static Vector sortStringList ( Vector list ) {
	int[] keys = Astro.sortStringKey(list);

 	Vector sorted_list = new Vector();
	for (int i = 0 ; i < list.size() ; i++)
	    sorted_list.addElement(list.elementAt(keys[i]));

	return sorted_list;
    }

    public static int[] sortStringKey ( Vector list ) {
	int[] keys = new int[list.size()];
	for (int i = 0 ; i < list.size() ; i++)
	    keys[i] = i;

	for (int i = 1 ; i < list.size() ; i++) {
	    int j = i - 1;
	    while (j >= 0  &&  ((String)list.elementAt(keys[j])).compareTo((String)list.elementAt(keys[j+1])) > 0) {
		int k = keys[j];
		keys[j] = keys[j+1];
		keys[j+1] = k;
		j--;
	    }
	}

	return keys;
    }

    public static String replaceString ( String target, String src, String dest ) {
	String result = target;

	int length = src.length();
	if (length > 0) {
	    int p = target.indexOf(src);
	    while (p >= 0) {
		result = target.substring(0, p) + dest + target.substring(p + length);
		target = result;
		p = target.indexOf(src);
	    }
	}

	return result;
    }

    public static String toString ( int val, int columns ) {
	return toString(val, columns, ' ');
    }

    public static String toStringZeroPadding ( int val, int columns ) {
	return toString(val, columns, '0');
    }

    private static String toString ( int val, int columns, char ch ) {
	int i;
	String s = "";

	for (i = 0 ; i < columns ; i++)
	    s += ch;
	s += String.valueOf(val);
	if (String.valueOf(val).length() > columns)
	    return String.valueOf(val);
	else {
	    s = s.substring(s.length() - columns, s.length());
	    if (ch == '0'  &&  s.indexOf('-') > 0)
		s = "-" + s.substring(1).replace('-', '0');
	    return s;
	}
    }

    public static String toString ( double val, int columns, int period ) {
	return toString(val, columns, period, ' ');
    }

    public static String toStringZeroPadding ( double val, int columns, int period ) {
	return toString(val, columns, period, '0');
    }

    private static String toString ( double val, int columns, int period, char ch ) {
	int i;
	String s = "";

	int after_period = columns - period - 1;

	double geta = 0.5;
	for (i = 0 ; i < after_period ; i++)
	    geta /= 10.0;

	if (val < 0.0)
	    val -= geta;
	else
	    val += geta;

	for (i = 0 ; i < columns ; i++)
	    s += ch;
	s += String.valueOf(val);
	if (s.indexOf('.') < 0)
	    s += '.';
	for (i = 0 ; i < columns ; i++)
	    s += '0';

	if (String.valueOf(val).indexOf('.') > period) {
	    columns += String.valueOf(val).indexOf('.') - period;
	    period = String.valueOf(val).indexOf('.');
	}

	int p = s.indexOf('.');
	if (after_period < 0)
	    s = s.substring(p - columns, p);
	else
	    s = s.substring(p - period, p - period + columns);

	if (ch == '0'  &&  s.indexOf('-') > 0)
	    s = "-" + s.substring(1).replace('-', '0');

	return s;
    }

    public static double stringValueOf ( String s ) {
	double value = 0.0;
	double keta = 1.0;

	for (int i = 0 ; i < s.length() ; i++) {
	    int p = (int)s.charAt(i);
	    value += (double)p * keta;
	    keta /= 100.0;
	}

	return value;
    }
}
