/*
 * CatalogList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class CatalogList extends StarList {
    private Coor center_coor;
    private AreaManager area_manager;

    private static final int RECORD_DETAIL = 0;
    private static final int RECORD_SIMPLE = 1;
    private static final int RECORD_NULL = 2;

    private int record_mode;

    // distance between two elements becomes in degree after multiplied by this
    private double unit_to_deg;

    public CatalogList ( ) {
	super();

	record_mode = RECORD_DETAIL;
	unit_to_deg = 1.0;
    }

    public CatalogList ( StarList l ) {
	super(l);

	record_mode = RECORD_DETAIL;
	unit_to_deg = 1.0;
    }

    public CatalogList ( StarList l, int count ) {
	super(l, count);

	record_mode = RECORD_DETAIL;
	unit_to_deg = 1.0;
    }

    public void setUnit ( double u ) {
	unit_to_deg = u;
    }

    public double getUnit ( ) {
	return unit_to_deg;
    }

    public void rescaleUnit ( double u ) {
	rescaleXY(unit_to_deg / u);
	unit_to_deg = u;
    }

    // set mode to load catalog data
    public void setModeNull ( ) {
	record_mode = RECORD_NULL;
    }
    public void setModeSimple ( ) {
	record_mode = RECORD_SIMPLE;
    }
    public void setModeDetail ( ) {
	record_mode = RECORD_DETAIL;
    }
    public boolean isModeNull ( ) {
	return record_mode == RECORD_NULL;
    }
    public boolean isModeSimple ( ) {
	return record_mode == RECORD_SIMPLE;
    }
    public boolean isModeDetail ( ) {
	return record_mode == RECORD_DETAIL;
    }

    // re-calculate (x,y) of all elements as R.A. and Decl. in argument is center
    public void mapElement ( Coor original_center_coor, Coor new_center_coor, double new_fov ) {
	for (int i = 0 ; i < size() ; i++) {
	    CatalogData sd = (CatalogData)elementAt(i);
	    Position pos = new Position(elementAt(i).getPosition());
	    pos.rescale(unit_to_deg);
	    CoorMapping coor = new CoorMapping(original_center_coor, pos);
	    pos = coor.mappingXY(new_center_coor);
	    pos.rescale(1.0 / new_fov);
	    sd.setPosition(pos);
	}

	unit_to_deg = new_fov;
    }

    // re-calculate (x,y) of all elements as R.A. and Decl. in argument is center
    // return only data within the range
    public CatalogList selectElementAsReference ( Coor original_center_coor, Coor new_center_coor, double select_fov ) {
	return selectElement(original_center_coor, new_center_coor, select_fov, false);
    }

    public CatalogList selectElementAsCopy ( Coor original_center_coor, Coor new_center_coor, double select_fov ) {
	return selectElement(original_center_coor, new_center_coor, select_fov, true);
    }

    private CatalogList selectElement ( Coor original_center_coor, Coor new_center_coor, double select_fov, boolean copy_flag ) {
	CatalogList list = new CatalogList();
	CatalogData sd = new CatalogData();
	for (int i = 0 ; i < size() ; i++) {
	    if (copy_flag)
		sd = new CatalogData(elementAt(i));
	    else
		sd = (CatalogData)elementAt(i);
	    list.addElement(sd);
	}
	list.setUnit(getUnit());

	list.mapElement(original_center_coor, new_center_coor, select_fov);

	for (int i = 0 ; i < list.size() ; i++) {
	    sd = (CatalogData)elementAt(i);
	    if (! (-0.5 <= sd.getX()  &&  sd.getX() <= 0.5  &&  -0.5 <= sd.getY()  &&  sd.getY() <= 0.5)) {
		list.removeElementAt(i);
		i--;
	    }
	}

	return list;
    }

    public void restrictArea ( Coor coor, double fov ) {
	center_coor = coor;
	area_manager = new AreaManager(coor, fov);
    }

    public boolean overlapArea ( Coor start_coor, Coor end_coor ) {
	return area_manager.overlapArea(start_coor, end_coor);
    }

    public boolean inArea ( Coor coor ) {
	return area_manager.inArea(coor);
    }

    public void addStar ( Coor center_coor, CoorMapping coor, double mag, String id, String record ) {
	Position pos = coor.mappingXY(center_coor);

	CatalogData sd = new CatalogData();
	sd.setCoor(coor);
	sd.setX(pos.getX() / unit_to_deg);
	sd.setY(pos.getY() / unit_to_deg);
	sd.setMag(mag);
	sd.setID(id);
	sd.setRecord(record);
	addElement(sd);
    }

    public void unifyDuplication ( ) {
	// mean position and magnitude
	for (int i = 0 ; i < size() ; i++) {
	    CatalogData sd = (CatalogData)elementAt(i);
	    double x = sd.getX();
	    double y = sd.getY();
	    double mag = sd.getMag();
	    int count = 1;

	    CatalogData sd2 = (CatalogData)sd.getFirstAppendix();
	    while (sd2 != null) {
		x += sd2.getX();
		y += sd2.getY();
		mag += sd2.getMag();
		count++;
		sd2 = (CatalogData)sd.getNextAppendix();
	    }

	    if (count > 1) {
		sd.setX(x / (double)count);
		sd.setY(y / (double)count);
		sd.setMag(mag / (double)count);
	    }
	}
    }

    // to be called just after loading
    public void unifyDuplication ( double rdeg_threshold, double mag_threshold ) {
	double r_threshold = rdeg_threshold / unit_to_deg;

	double mesh_unit = 0.01;
	if (r_threshold * 2.0 > mesh_unit)
	    mesh_unit = r_threshold * 2.0;

	StarListIndex index = new StarListIndex(this, new Position(-0.5, -0.5), new Position(0.5, 0.5), mesh_unit, new MapFunction());
	index.unify(new MapFunction(), r_threshold, mag_threshold);

	removeAllElements();
	add(index.getAllList());

	unifyDuplication();
    }

    public CatalogData createCatalogData ( String s ) {
	return null;
    }
}
