/*
 * ImageGray.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.awt.*;
import java.awt.image.*;

public class ImageGray {
    private ImageBuffer ib;

    public ImageGray ( ) {
	ib = new ImageBuffer();
    }

    public ImageGray ( Size size ) throws Exception {
	allocate(size);
    }

    public ImageGray ( ImageGray image ) {
	ib = new ImageBuffer(image.ib);
    }

    public ImageGray ( Image image, Size size ) throws Exception {
	ImageRGB rgb_image;
	try {
	    rgb_image = new ImageRGB(image, size);
	} catch (Exception e) {
	    throw e;
	}

	try {
	    allocate(size);
	} catch ( Exception e ) {
	    throw e;
	}

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, rgb_image.getGrayValue(pt));
	    }
	}
    }

    protected void allocate ( Size size ) throws Exception {
	try {
	    ib = new ImageBuffer(size);
	} catch (Exception e) {
	    throw e;
	}
    }

    public Size getSize ( ) {
	return ib.getSize();
    }

    public int getWidth ( ) {
	return ib.getWidth();
    }

    public int getHeight ( ) {
	return ib.getHeight();
    }

    public void setValue ( Point pt, double val ) {
	ib.setValue(pt, val);
    }

    public double getValue ( Point pt ) {
	return ib.getValue(pt);
    }

    public double getValueByFolding ( Point pt ) {
	Point pt2 = new Point(pt);
	if (pt2.x < 0)
	    pt2.x = - pt2.x;
	if (pt2.x >= getWidth())
	    pt2.x = 2 * (getWidth() - 1) - pt2.x;
	if (pt2.y < 0)
	    pt2.y = - pt2.y;
	if (pt2.y >= getHeight())
	    pt2.y = 2 * (getHeight() - 1) - pt2.y;
	return ib.getValue(pt2);
    }

    public void fill ( double value ) {
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, value);
	    }
	}
    }

    public void addValue ( double value ) {
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, getValue(pt) + value);
	    }
	}
    }

    public void multiplyValue ( double value ) {
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, getValue(pt) * value);
	    }
	}
    }

    public MinMaxAverage calculateMinMaxAverage ( ) {
	double min = 0.0;
	double max = 0.0;
	double average = 0.0;

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (pt.equals(new Point(0, 0))) {
		    min = getValue(pt);
		    max = getValue(pt);
		} else {
		    if (min > getValue(pt))
			min = getValue(pt);
		    if (max < getValue(pt))
			max = getValue(pt);
		}
		average += getValue(pt);
	    }
	}

	if (getWidth() * getHeight() > 0)
	    average /= (double)(getWidth() * getHeight());

	return new MinMaxAverage(min, max, average, getWidth() * getHeight());
    }

    public Image getImage ( ) throws Exception {
	MinMaxAverage mma = calculateMinMaxAverage();
	try {
	    return getImage(mma.getMin(), mma.getMax());
	} catch (Exception e) {
	    throw e;
	}
    }

    public Image getImage ( double min, double max ) throws Exception {
	int p[] = new int[getWidth() * getHeight()];
	if (p == null) {
	    System.err.println("Out of memory. Cannot allocate Image.");
	    throw new Exception();
	}

	int i = 0;
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		p[i] = (int)((getValue(pt) - min) / (max - min) * 256);
		if (p[i] < 0)
		    p[i] = 0;
		if (p[i] > 255)
		    p[i] = 255;
		p[i] = (255 << 24) | (p[i] << 16) | (p[i] << 8) | p[i];
		i++;
	    }
	}
	Image image = Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(getWidth(), getHeight(), p, 0, getWidth()));
	return image;
    }

    public ImageGray resize ( Size new_size ) throws Exception {
	ImageGray image;
	try {
	    image = new ImageGray(new_size);
	} catch (Exception e) {
	    throw e;
	}

	Point dpt = new Point((getHeight() - new_size.getHeight()) / 2, (getWidth() - new_size.getWidth()) / 2);
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < new_size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < new_size.getWidth() ; pt.x++) {
		image.setValue(pt, getValue(new Point(pt.x + dpt.x, pt.y + dpt.y)));
	    }
	}

	return image;
    }

    public ImageGray smooth_average ( int size ) throws Exception {
	return smooth_average(size, 1.0);
    }

    public ImageGray smooth_average ( int size, double weight ) throws Exception {
	ImageGray image;
	ImageGray image2;
	try {
	    image = new ImageGray(getSize());
	    image2 = new ImageGray(getSize());
	} catch (Exception e) {
	    throw e;
	}

	double val_count = (double)size + (weight - 1.0);

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    double val = 0.0;
	    for (pt.x = - (size - 1) / 2 - 1 ; pt.x < size / 2 ; pt.x++)
		val += getValueByFolding(pt);

	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		val -= getValueByFolding(new Point(pt.x - (size - 1) / 2 - 1, pt.y));
		val += getValueByFolding(new Point(pt.x + size / 2, pt.y));

		double val2 = val;
		val2 += (weight - 1.0) * getValue(pt);
		image.setValue(pt, val2 / val_count);
	    }
	}

	for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
	    double val = 0.0;
	    for (pt.y = - (size - 1) / 2 - 1 ; pt.y < size / 2 ; pt.y++)
		val += image.getValueByFolding(pt);

	    for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
		val -= image.getValueByFolding(new Point(pt.x, pt.y - (size - 1) / 2 - 1));
		val += image.getValueByFolding(new Point(pt.x, pt.y + size / 2));

		double val2 = val;
		val2 += (weight - 1.0) * image.getValue(pt);
		image2.setValue(pt, val2 / val_count);
	    }
	}

	return image2;
    }

    // expansion and reduction
    public ImageGray rescale ( Size new_size ) throws Exception {
	ImageGray image;
	ImageGray image2;
	try {
	    image = new ImageGray(new Size(new_size.getWidth(), getHeight()));
	    image2 = new ImageGray(new_size);
	} catch (Exception e) {
	    throw e;
	}

	double rx = (double)new_size.getWidth() / (double)getWidth();
	double ry = (double)new_size.getHeight() / (double)getHeight();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < new_size.getWidth() ; pt.x++)
		image.setValue(pt, 0.0);

	    Point old_pt = new Point(0, pt.y);
	    pt.x = 0;
	    double phase = 0.0;
	    for (old_pt.x = 0 ; old_pt.x < getWidth() ; old_pt.x++) {
		if (phase + rx <= 1.0) {
		    image.setValue(pt, image.getValue(pt) + getValue(old_pt) * rx);
		    phase += rx;
		} else {
		    image.setValue(pt, image.getValue(pt) + getValue(old_pt) * (1.0 - phase));
		    double r = rx - (1.0 - phase);
		    pt.x++;
		    phase = 0.0;
		    while (r >= 1.0) {
			image.setValue(pt, getValue(old_pt));
			pt.x++;
			r -= 1.0;
		    }
		    if (r > 0.0) {
			image.setValue(pt, getValue(old_pt) * r);
			phase = r;
		    }
		}
	    }
	}

	for (pt.x = 0 ; pt.x < new_size.getWidth() ; pt.x++) {
	    for (pt.y = 0 ; pt.y < new_size.getHeight() ; pt.y++)
		image2.setValue(pt, 0.0);

	    Point old_pt = new Point(pt.x, 0);
	    pt.y = 0;
	    double phase = 0.0;
	    for (old_pt.y = 0 ; old_pt.y < getHeight() ; old_pt.y++) {
		if (phase + ry <= 1.0) {
		    image2.setValue(pt, image2.getValue(pt) + image.getValue(old_pt) * ry);
		    phase += ry;
		} else {
		    image2.setValue(pt, image2.getValue(pt) + image.getValue(old_pt) * (1.0 - phase));
		    double r = ry - (1.0 - phase);
		    pt.y++;
		    phase = 0.0;
		    while (r >= 1.0) {
			image2.setValue(pt, image.getValue(old_pt));
			pt.y++;
			r -= 1.0;
		    }
		    if (r > 0.0) {
			image2.setValue(pt, image.getValue(old_pt) * r);
			phase = r;
		    }
		}
	    }
	}

	return image2;
    }

    public ImageGray rotate ( double theta ) throws Exception {
	int resolution = 2;

	double sin_t = Math.sin(theta * Astro.RAD);
	double cos_t = Math.cos(theta * Astro.RAD);
	Size new_size = new Size((int)(Math.abs((double)getWidth() * cos_t + (double)getHeight() * sin_t) + 0.5), (int)(Math.abs((double)getWidth() * sin_t + (double)getHeight() * cos_t) + 0.5));

	ImageGray image;
	try {
	    image = new ImageGray(new_size);
	} catch (Exception e) {
	    throw e;
	}

	Point pt = new Point(0, 0);
	Point opt = new Point(0, 0);
	int i, j, count;
	double dx, dy;
	double val;

	for (pt.y = 0 ; pt.y < new_size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < new_size.getWidth() ; pt.x++) {
		val = 0.0;
		count = 0;
		for (i = 0 ; i < resolution ; i++) {
		    dy = (double)(i + 1) / (double)(resolution + 1);
		    for (j = 0 ; j < resolution ; j++) {
			dx = (double)(j + 1) / (double)(resolution + 1);
			opt.x = (int)(((double)pt.x + dx - (double)new_size.getWidth() / 2.0) * cos_t - ((double)pt.y + dy - (double)new_size.getHeight() / 2.0) * sin_t + (double)getWidth() / 2.0 + 0.5);
			opt.y = (int)(((double)pt.x + dx - (double)new_size.getWidth() / 2.0) * sin_t + ((double)pt.y + dy - (double)new_size.getHeight() / 2.0) * cos_t + (double)getHeight() / 2.0 + 0.5);
			if (0 <= opt.x  &&  opt.x < getWidth()  &&  0 <= opt.y  &&  opt.y < getHeight()) {
			    val += getValue(opt);
			    count++;
			}
		    }
		}
		if (count > 0)
		    val /= (double)count;
		image.setValue(pt, val);
	    }
	}

	return image;
    }

    // left <-> right
    public void mirrorHorizontal ( ) {
	Point pt = new Point(0, 0);
	Point pt2 = new Point(0, 0);
	for (pt.y = 0, pt2.y = 0 ; pt.y < getHeight() ; pt.y++, pt2.y++) {
	    for (pt.x = 0, pt2.x = getWidth() - 1 ; pt.x < getWidth() / 2 ; pt.x++, pt2.x--) {
		double val = getValue(pt);
		setValue(pt, getValue(pt2));
		setValue(pt2, val);
	    }
	}
    }

    // upside down
    public void mirrorVertical ( ) {
	Point pt = new Point(0, 0);
	Point pt2 = new Point(0, 0);
	for (pt.x = 0, pt2.x = 0 ; pt.x < getWidth() ; pt.x++, pt2.x++) {
	    for (pt.y = 0, pt2.y = getHeight() - 1 ; pt.y < getHeight() / 2 ; pt.y++, pt2.y--) {
		double val = getValue(pt);
		setValue(pt, getValue(pt2));
		setValue(pt2, val);
	    }
	}
    }

    // gamma correction to fit log curve
    public void gammaLog ( ) {
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, Math.log(getValue(pt)));
	    }
	}
    }

    // gamma correction to fit sqrt curve
    public void gammaSqrt ( ) {
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		setValue(pt, Math.sqrt(getValue(pt)));
	    }
	}
    }

    public Histogram histogram ( ) {
	return histogram(256);
    }

    private Histogram histogram ( int steps ) {
	Histogram hist = new Histogram(steps);

	MinMaxAverage mma = calculateMinMaxAverage();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		int g = (int)((getValue(pt) - mma.getMin()) / (mma.getMax() - mma.getMin()) * (double)steps);
		hist.addAt(g);
	    }
	}

	hist.accumulate();

	return hist;
    }

    // median filter
    public ImageGray median ( int filter_size ) throws Exception {
	ImageGray image = new ImageGray(getSize());

	if (filter_size <= 0)
	    filter_size = 1;

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		int count = 0;
		Point d_pt = new Point(0, 0);
		for (d_pt.y = - filter_size / 2 ; d_pt.y <= filter_size / 2 ; d_pt.y++) {
		    for (d_pt.x = - filter_size / 2 ; d_pt.x <= filter_size / 2 ; d_pt.x++) {
			if (0 <= pt.y + d_pt.y  &&  pt.y + d_pt.y < getHeight()  &&  0 <= pt.x + d_pt.x  &&  pt.x + d_pt.x < getWidth())
			    count++;
		    }
		}

		Array array = new Array(count);

		count = 0;
		for (d_pt.y = - filter_size / 2 ; d_pt.y <= filter_size / 2 ; d_pt.y++) {
		    for (d_pt.x = - filter_size / 2 ; d_pt.x <= filter_size / 2 ; d_pt.x++) {
			if (0 <= pt.y + d_pt.y  &&  pt.y + d_pt.y < getHeight()  &&  0 <= pt.x + d_pt.x  &&  pt.x + d_pt.x < getWidth())
			    array.set(count++, getValue(new Point(pt.x + d_pt.x, pt.y + d_pt.y)));
		    }
		}

		array.sortAscendant();

		image.setValue(pt, array.getValue(count / 2));
	    }
	}

	return image;
    }
}
