/*
 * JulianDay.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.*;

public class JulianDay {
    private double julian_day;
    private int year;
    private int month;
    private double day;

    private static final String month_name[] = { "", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" };
    private static final String month_code[] = { "", "Jan.", "Feb.", "Mar.", "Apr.", "May", "June", "July", "Aug.", "Sept.", "Oct.", "Nov.", "Dec." };

    // Julian day -> year, month, day
    private void update ( ) {
	int i, p, q, r, s, t, t_old;
	JulianDay date;

	year = (int)(julian_day / 365.2421988 - 4712.0);
	date = new JulianDay(year, 1, 0.0);
	if (date.getJD() + 1.0 > julian_day) {
	    year--;
	    date = new JulianDay(year, 1, 0.0);
	}
	double jd0 = julian_day - date.getJD();

	i = 1;
	t = 0;
	do {
	    i++;
	    t_old = t;
	    p = i - 1;
	    q = (int)((i + 7) / 10);
	    r = (int)((double)year / 4.0 - (double)((int)(year / 4)) + 0.77);
	    s = (int)(0.55 * p - 0.33);
	    t = 30 * p + q * (s - r) + p * (1 - q);
	} while (jd0 - (double)t >= 1.0  &&  i < 13) ;

	month = i - 1;
	day = jd0 - (double)t_old;
    }

    public JulianDay ( ) {
	this(Astro.JULIUS_2000);
    }

    public JulianDay ( double jd ) {
	julian_day = jd;
	update();
    }

    public JulianDay ( int y, int m, double d ) {
	// year, month, day -> Julian day
	year = y;
	month = m;
	day = d;

	long a;

	if (m < 3) {
	    y -= 1;
	    m += 12;
	}

	if (y >= 1583  ||  (y == 1582  &&  (m >= 11  ||  (m == 10  &&  d >= 15.0)))){	// Gregorian calendar (since 1582 Oct. 15)
	    a = (long)(365.25 * (double)y) + (long)(y / 400) - (long)(y / 100) + (long)(30.59 * (double)(m - 2));
	    julian_day = (double)a + d + 1721088.5;
	} else {	// Julian calendar (until 1582 Oct. 4)
	    a = (long)(365.25 * (double)y) + (long)(30.59 * (double)(m - 2));
	    julian_day = (double)a + d + 1721086.5;
	    if (y < 0)
		julian_day -= 1.0;
	}
    }

    public JulianDay ( int y, int m, int d, int h, int min, double s ) {
	this(y, m, (double)d);

	double hms = (double)h / 24.0 + (double)min / 24.0 / 60.0 + s / 24.0 / 3600.0;
	day += hms;
	julian_day += hms;
    }

    public static JulianDay createJulianDay ( String str ) {
	StringTokenizer st = new StringTokenizer(str);
	int y = Integer.parseInt(st.nextToken());
	String month_code = st.nextToken();
	int m = 1;
	if ('0' <= month_code.charAt(0)  &&  month_code.charAt(0) <= '9') {
	    m = Integer.parseInt(month_code);
	} else {
	    for (m = 1 ; m <= 12 ; m++) {
		if (getMonthCode(m).substring(0,3).equalsIgnoreCase(month_code.substring(0,3)))
		    break;
	    }
	}
	String day = st.nextToken();

	if (! st.hasMoreTokens()) {
	    double d = Double.valueOf(day).doubleValue();
	    return new JulianDay(y, m, d);
	}

	int d = Integer.parseInt(day);

	int h, min, s;
	String hms = st.nextToken();
	if (hms.indexOf(':') >= 0) {
	    h = Integer.parseInt(hms.substring(0,2).trim());
	    min = Integer.parseInt(hms.substring(3,5).trim());
	    s = Integer.parseInt(hms.substring(6,8).trim());
	} else {
	    h = Integer.parseInt(hms);
	    min = Integer.parseInt(st.nextToken());
	    s = Integer.parseInt(st.nextToken());
	}

	return new JulianDay(y, m, d, h, min, s);
    }

    public int getYear ( ) {
	return year;
    }

    public int getMonth ( ) {
	return month;
    }

    public String getMonthName ( ) {
	return month_name[month];
    }

    public String getMonthCode ( ) {
	return month_code[month];
    }

    public static String getMonthName ( int m ) {
	return month_name[m];
    }

    public static String getMonthCode ( int m ) {
	return month_code[m];
    }

    public double getDate ( ) {
	return day;
    }

    public int getHour ( ) {
	return (int)((day - (int)day) * 24.0);
    }

    public int getMinute ( ) {
	return (int)((day - (int)day - getHour() / 24.0) * 24.0 * 60.0);
    }

    public double getSecond ( ) {
	return (day - (int)day - getHour() / 24.0 - getMinute() / 24.0 / 60.0) * 24.0 * 60.0 * 60.0;
    }

    public double getJD ( ) {
	return julian_day;
    }

    // return how many days exist in the month
    public static int getMaxDay ( int y, int m ) {
	int d = 0;

	switch (m) {
	    case 1:
	    case 3:
	    case 5:
	    case 7:
	    case 8:
	    case 10:
	    case 12:
		d = 31;
		break;
	    case 4:
	    case 6:
	    case 9:
	    case 11:
		d = 30;
		break;
	    case 2:
		if (y % 4 == 0  &&  (y % 100 != 0  ||  y % 400 == 0))
		    d = 29;
		else
		    d = 28;
		break;
	}

	return d;
    }

    public String toString ( ) {
	String s = "" + getYear() + " " + getMonthCode() + " " + (int)getDate() + "  " + Astro.toStringZeroPadding(getHour(), 2) + ":" + Astro.toStringZeroPadding(getMinute(), 2) + ":" + Astro.toStringZeroPadding((int)(getSecond() + 0.5), 2) + " UT";
	return s;
    }
}
